@@ -11,7 +11,7 @@ pub mod value;
 #[cfg(test)]
 mod dom {
     use super::value::{
-        Value, IOValue, NestedValue, PlainValue, Domain,
+        Value, IOValue, IOResult, NestedValue, PlainValue, Domain,
         PackedWriter,
     };
 
@@ -22,7 +22,7 @@ mod dom {
     }
 
     impl Domain for Dom {
-        fn from_preserves(v: IOValue) -> Self {
+        fn from_preserves(v: IOValue) -> IOResult<Self> {
             panic!("Cannot decode IOValue to Dom: {:?}", v);
         }
 
@@ -164,18 +164,18 @@ mod value_tests {
 
     #[test] fn float_mut() {
         let mut f = VV::from(1.0f32);
-        assert!(f.is_float());
-        *(f.as_float_mut().unwrap()) = 123.45;
+        assert!(f.is_f32());
+        *(f.as_f32_mut().unwrap()) = 123.45;
         assert_eq!(f, VV::from(123.45f32));
-        assert_eq!((f.as_float().unwrap() - 123.45f32).abs() < std::f32::EPSILON, true);
+        assert_eq!((f.as_f32().unwrap() - 123.45f32).abs() < std::f32::EPSILON, true);
     }
 
     #[test] fn double_mut() {
         let mut f = VV::from(1.0);
-        assert!(f.is_double());
-        *(f.as_double_mut().unwrap()) = 123.45;
+        assert!(f.is_f64());
+        *(f.as_f64_mut().unwrap()) = 123.45;
         assert_eq!(f, VV::from(123.45));
-        assert_eq!((f.as_double().unwrap() - 123.45).abs() < std::f64::EPSILON, true);
+        assert_eq!((f.as_f64().unwrap() - 123.45).abs() < std::f64::EPSILON, true);
     }
 
     #[test] fn signedinteger_mut() {
