@@ -16,6 +16,8 @@ pub use reader::Reader;
 pub use repr::AnnotatedValue;
 pub use repr::ArcValue;
 pub use repr::Domain;
+pub use repr::Double;
+pub use repr::Float;
 pub use repr::IOValue;
 pub use repr::Map;
 pub use repr::NestedValue;
@@ -33,6 +35,8 @@ pub use repr::FALSE;
 pub use repr::TRUE;
 pub use repr::EMPTY_SEQ;
 
+pub type IOResult<T> = std::result::Result<T, std::io::Error>;
+
 pub fn invert_map<A, B>(m: &Map<A, B>) -> Map<B, A>
     where A: Clone, B: Clone + Ord
 {
