@@ -5,8 +5,7 @@ pub mod writer;
 pub use reader::PackedReader;
 pub use writer::PackedWriter;
 
-use super::{Reader, IOValue};
-use super::reader::IOResult;
+use super::{Reader, IOValue, IOResult};
 
 pub fn from_bytes(bs: &[u8]) -> IOResult<IOValue> {
     PackedReader::decode_bytes(bs).demand_next(false)
