@@ -11,19 +11,31 @@ use std::marker::PhantomData;
 use std::ops::Index;
 use std::ops::IndexMut;
 use std::string::String;
+use std::sync::Arc;
 use std::vec::Vec;
 
 pub use std::collections::BTreeSet as Set;
 pub use std::collections::BTreeMap as Map;
 
+use super::IOResult;
 use super::signed_integer::SignedInteger;
 use crate::error::{Error, ExpectedKind, Received};
 
 pub trait Domain: Sized + Debug + Clone + Eq + Hash + Ord {
-    fn from_preserves(v: IOValue) -> Self;
+    fn from_preserves(v: IOValue) -> IOResult<Self>;
     fn as_preserves(&self) -> IOValue;
 }
 
+impl<D: Domain> Domain for Arc<D> {
+    fn from_preserves(v: IOValue) -> IOResult<Self> {
+        Ok(Arc::new(D::from_preserves(v)?))
+    }
+
+    fn as_preserves(&self) -> IOValue {
+        D::as_preserves(self)
+    }
+}
+
 pub trait NestedValue<D: Domain>: Sized + Debug + Clone + Eq + Hash + Ord {
     fn wrap(anns: Annotations<Self, D>, v: Value<Self, D>) -> Self;
 
@@ -39,19 +51,19 @@ pub trait NestedValue<D: Domain>: Sized + Debug + Clone + Eq + Hash + Ord {
         self.value().fmt(f)
     }
 
-    fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> M
+    fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> IOResult<M>
     where
-        F: Fn(&D) -> Value<M, E>
+        F: Fn(&D) -> IOResult<Value<M, E>>
     {
-        M::wrap(self.annotations().copy_via(f), self.value().copy_via(f))
+        Ok(M::wrap(self.annotations().copy_via(f)?, self.value().copy_via(f)?))
     }
 
     fn to_io_value(&self) -> IOValue {
-        self.copy_via(&|d| d.as_preserves().value().clone())
+        self.copy_via(&|d| Ok(d.as_preserves().value().clone())).unwrap()
     }
 
-    fn from_io_value(v: &IOValue) -> Self {
-        v.copy_via(&|d| Value::Domain(D::from_preserves(d.clone())))
+    fn from_io_value(v: &IOValue) -> IOResult<Self> {
+        v.copy_via(&|d| Ok(Value::Domain(D::from_preserves(d.clone())?)))
     }
 }
 
@@ -205,7 +217,9 @@ impl Eq for Double {}
 impl<N: NestedValue<D>, D: Domain> From<bool> for Value<N, D> { fn from(v: bool) -> Self { Value::Boolean(v) } }
 
 impl<N: NestedValue<D>, D: Domain> From<f32> for Value<N, D> { fn from(v: f32) -> Self { Value::Float(Float::from(v)) } }
+impl<N: NestedValue<D>, D: Domain> From<&Float> for Value<N, D> { fn from(v: &Float) -> Self { Value::Float(v.clone()) } }
 impl<N: NestedValue<D>, D: Domain> From<f64> for Value<N, D> { fn from(v: f64) -> Self { Value::Double(Double::from(v)) } }
+impl<N: NestedValue<D>, D: Domain> From<&Double> for Value<N, D> { fn from(v: &Double) -> Self { Value::Double(v.clone()) } }
 
 impl<N: NestedValue<D>, D: Domain> From<u8> for Value<N, D> { fn from(v: u8) -> Self { Value::from(i128::from(v)) } }
 impl<N: NestedValue<D>, D: Domain> From<i8> for Value<N, D> { fn from(v: i8) -> Self { Value::from(i128::from(v)) } }
@@ -241,13 +255,13 @@ impl<N: NestedValue<D>, D: Domain> Debug for Value<N, D> {
             Value::Float(Float(v)) => write!(f, "{:?}f", v),
             Value::Double(Double(v)) => write!(f, "{:?}", v),
             Value::SignedInteger(v) => write!(f, "{}", v),
-            Value::String(ref v) => write!(f, "{:?}", v), // TODO: proper escaping!
-            Value::ByteString(ref v) => {
+            Value::String(v) => write!(f, "{:?}", v), // TODO: proper escaping!
+            Value::ByteString(v) => {
                 f.write_str("#x\"")?;
                 for b in v { write!(f, "{:02x}", b)? }
                 f.write_str("\"")
             }
-            Value::Symbol(ref v) => {
+            Value::Symbol(v) => {
                 // TODO: proper escaping!
                 if v.is_empty() {
                     f.write_str("||")
@@ -255,7 +269,7 @@ impl<N: NestedValue<D>, D: Domain> Debug for Value<N, D> {
                     write!(f, "{}", v)
                 }
             }
-            Value::Record(ref r) => {
+            Value::Record(r) => {
                 f.write_str("<")?;
                 r.label().fmt(f)?;
                 for v in r.fields() {
@@ -264,13 +278,13 @@ impl<N: NestedValue<D>, D: Domain> Debug for Value<N, D> {
                 }
                 f.write_str(">")
             }
-            Value::Sequence(ref v) => v.fmt(f),
-            Value::Set(ref v) => {
+            Value::Sequence(v) => v.fmt(f),
+            Value::Set(v) => {
                 f.write_str("#")?;
                 f.debug_set().entries(v.iter()).finish()
             }
-            Value::Dictionary(ref v) => f.debug_map().entries(v.iter()).finish(),
-            Value::Domain(ref d) => write!(f, "#!{:?}", d),
+            Value::Dictionary(v) => f.debug_map().entries(v.iter()).finish(),
+            Value::Domain(d) => write!(f, "#!{:?}", d),
         }
     }
 }
@@ -291,15 +305,15 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_boolean(&self) -> Option<bool> {
-        if let Value::Boolean(b) = *self {
-            Some(b)
+        if let Value::Boolean(b) = self {
+            Some(*b)
         } else {
             None
         }
     }
 
     pub fn as_boolean_mut(&mut self) -> Option<&mut bool> {
-        if let Value::Boolean(ref mut b) = *self {
+        if let Value::Boolean(b) = self {
             Some(b)
         } else {
             None
@@ -314,56 +328,88 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
         self.as_float().is_some()
     }
 
-    pub fn as_float(&self) -> Option<f32> {
-        if let Value::Float(Float(f)) = *self {
+    pub fn as_float(&self) -> Option<&Float> {
+        if let Value::Float(f) = self {
             Some(f)
         } else {
             None
         }
     }
 
-    pub fn as_float_mut(&mut self) -> Option<&mut f32> {
-        if let Value::Float(Float(ref mut f)) = *self {
+    pub fn as_float_mut(&mut self) -> Option<&mut Float> {
+        if let Value::Float(f) = self {
             Some(f)
         } else {
             None
         }
     }
 
-    pub fn to_float(&self) -> Result<f32, Error> {
+    pub fn to_float(&self) -> Result<&Float, Error> {
         self.as_float().ok_or_else(|| self.expected(ExpectedKind::Float))
     }
 
+    pub fn is_f32(&self) -> bool {
+        self.is_float()
+    }
+
+    pub fn as_f32(&self) -> Option<f32> {
+        self.as_float().map(|f| f.0)
+    }
+
+    pub fn as_f32_mut(&mut self) -> Option<&mut f32> {
+        self.as_float_mut().map(|f| &mut f.0)
+    }
+
+    pub fn to_f32(&self) -> Result<f32, Error> {
+        self.to_float().map(|f| f.0)
+    }
+
     pub fn is_double(&self) -> bool {
         self.as_double().is_some()
     }
 
-    pub fn as_double(&self) -> Option<f64> {
-        if let Value::Double(Double(f)) = *self {
+    pub fn as_double(&self) -> Option<&Double> {
+        if let Value::Double(f) = self {
             Some(f)
         } else {
             None
         }
     }
 
-    pub fn as_double_mut(&mut self) -> Option<&mut f64> {
-        if let Value::Double(Double(ref mut f)) = *self {
+    pub fn as_double_mut(&mut self) -> Option<&mut Double> {
+        if let Value::Double(f) = self {
             Some(f)
         } else {
             None
         }
     }
 
-    pub fn to_double(&self) -> Result<f64, Error> {
+    pub fn to_double(&self) -> Result<&Double, Error> {
         self.as_double().ok_or_else(|| self.expected(ExpectedKind::Double))
     }
 
+    pub fn is_f64(&self) -> bool {
+        self.is_double()
+    }
+
+    pub fn as_f64(&self) -> Option<f64> {
+        self.as_double().map(|f| f.0)
+    }
+
+    pub fn as_f64_mut(&mut self) -> Option<&mut f64> {
+        self.as_double_mut().map(|f| &mut f.0)
+    }
+
+    pub fn to_f64(&self) -> Result<f64, Error> {
+        self.to_double().map(|f| f.0)
+    }
+
     pub fn is_signedinteger(&self) -> bool {
         self.as_signedinteger().is_some()
     }
 
     pub fn as_signedinteger(&self) -> Option<&SignedInteger> {
-        if let Value::SignedInteger(ref n) = *self {
+        if let Value::SignedInteger(n) = self {
             Some(n)
         } else {
             None
@@ -371,7 +417,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_signedinteger_mut(&mut self) -> Option<&mut SignedInteger> {
-        if let Value::SignedInteger(ref mut n) = *self {
+        if let Value::SignedInteger(n) = self {
             Some(n)
         } else {
             None
@@ -498,7 +544,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_string(&self) -> Option<&String> {
-        if let Value::String(ref s) = *self {
+        if let Value::String(s) = self {
             Some(s)
         } else {
             None
@@ -506,7 +552,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_string_mut(&mut self) -> Option<&mut String> {
-        if let Value::String(ref mut s) = *self {
+        if let Value::String(s) = self {
             Some(s)
         } else {
             None
@@ -522,7 +568,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_bytestring(&self) -> Option<&Vec<u8>> {
-        if let Value::ByteString(ref s) = *self {
+        if let Value::ByteString(s) = self {
             Some(s)
         } else {
             None
@@ -530,7 +576,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_bytestring_mut(&mut self) -> Option<&mut Vec<u8>> {
-        if let Value::ByteString(ref mut s) = *self {
+        if let Value::ByteString(s) = self {
             Some(s)
         } else {
             None
@@ -550,7 +596,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_symbol(&self) -> Option<&String> {
-        if let Value::Symbol(ref s) = *self {
+        if let Value::Symbol(s) = self {
             Some(s)
         } else {
             None
@@ -558,7 +604,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_symbol_mut(&mut self) -> Option<&mut String> {
-        if let Value::Symbol(ref mut s) = *self {
+        if let Value::Symbol(s) = self {
             Some(s)
         } else {
             None
@@ -580,7 +626,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_record(&self, arity: Option<usize>) -> Option<&Record<N>> {
-        if let Value::Record(ref r) = *self {
+        if let Value::Record(r) = self {
             match arity {
                 Some(expected) if r.arity() == expected => Some(r),
                 Some(_other) => None,
@@ -592,7 +638,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_record_mut(&mut self, arity: Option<usize>) -> Option<&mut Record<N>> {
-        if let Value::Record(ref mut r) = *self {
+        if let Value::Record(r) = self {
             match arity {
                 Some(expected) if r.arity() == expected => Some(r),
                 Some(_other) => None,
@@ -628,7 +674,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     pub fn as_simple_record(&self, label: &str, arity: Option<usize>) -> Option<&[N]> {
         self.as_record(arity).and_then(|r| {
             match r.label().value() {
-                Value::Symbol(ref s) if s == label => Some(r.fields()),
+                Value::Symbol(s) if s == label => Some(r.fields()),
                 _ => None
             }
         })
@@ -656,7 +702,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_sequence(&self) -> Option<&Vec<N>> {
-        if let Value::Sequence(ref s) = *self {
+        if let Value::Sequence(s) = self {
             Some(s)
         } else {
             None
@@ -664,7 +710,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_sequence_mut(&mut self) -> Option<&mut Vec<N>> {
-        if let Value::Sequence(ref mut s) = *self {
+        if let Value::Sequence(s) = self {
             Some(s)
         } else {
             None
@@ -680,7 +726,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_set(&self) -> Option<&Set<N>> {
-        if let Value::Set(ref s) = *self {
+        if let Value::Set(s) = self {
             Some(s)
         } else {
             None
@@ -688,7 +734,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_set_mut(&mut self) -> Option<&mut Set<N>> {
-        if let Value::Set(ref mut s) = *self {
+        if let Value::Set(s) = self {
             Some(s)
         } else {
             None
@@ -704,7 +750,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_dictionary(&self) -> Option<&Map<N, N>> {
-        if let Value::Dictionary(ref s) = *self {
+        if let Value::Dictionary(s) = self {
             Some(s)
         } else {
             None
@@ -712,7 +758,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_dictionary_mut(&mut self) -> Option<&mut Map<N, N>> {
-        if let Value::Dictionary(ref mut s) = *self {
+        if let Value::Dictionary(s) = self {
             Some(s)
         } else {
             None
@@ -728,7 +774,7 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
     }
 
     pub fn as_embedded(&self) -> Option<&D> {
-        if let Value::Domain(ref d) = *self {
+        if let Value::Domain(d) = self {
             Some(d)
         } else {
             None
@@ -739,26 +785,30 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
         self.as_embedded().ok_or_else(|| self.expected(ExpectedKind::Embedded))
     }
 
-    pub fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> Value<M, E>
+    pub fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> IOResult<Value<M, E>>
     where
-        F: Fn(&D) -> Value<M, E>
+        F: Fn(&D) -> IOResult<Value<M, E>>
     {
-        match self {
+        Ok(match self {
             Value::Boolean(b) => Value::Boolean(*b),
             Value::Float(f) => Value::Float(f.clone()),
             Value::Double(d) => Value::Double(d.clone()),
-            Value::SignedInteger(ref n) => Value::SignedInteger(n.clone()),
-            Value::String(ref s) => Value::String(s.clone()),
-            Value::ByteString(ref v) => Value::ByteString(v.clone()),
-            Value::Symbol(ref v) => Value::Symbol(v.clone()),
-            Value::Record(ref r) =>
-                Value::Record(Record(r.fields_vec().iter().map(|a| a.copy_via(f)).collect())),
-            Value::Sequence(ref v) => Value::Sequence(v.iter().map(|a| a.copy_via(f)).collect()),
-            Value::Set(ref v) => Value::Set(v.iter().map(|a| a.copy_via(f)).collect()),
-            Value::Dictionary(ref v) =>
-                Value::Dictionary(v.iter().map(|(a,b)| (a.copy_via(f), b.copy_via(f))).collect()),
-            Value::Domain(d) => f(d),
-        }
+            Value::SignedInteger(n) => Value::SignedInteger(n.clone()),
+            Value::String(s) => Value::String(s.clone()),
+            Value::ByteString(v) => Value::ByteString(v.clone()),
+            Value::Symbol(v) => Value::Symbol(v.clone()),
+            Value::Record(r) =>
+                Value::Record(Record(r.fields_vec().iter().map(|a| a.copy_via(f))
+                                     .collect::<Result<Vec<_>, _>>()?)),
+            Value::Sequence(v) => Value::Sequence(v.iter().map(|a| a.copy_via(f))
+                                                      .collect::<IOResult<Vec<_>>>()?),
+            Value::Set(v) => Value::Set(v.iter().map(|a| a.copy_via(f))
+                                            .collect::<IOResult<Set<_>>>()?),
+            Value::Dictionary(v) =>
+                Value::Dictionary(v.iter().map(|(a,b)| Ok((a.copy_via(f)?, b.copy_via(f)?)))
+                                  .collect::<IOResult<Map<_, _>>>()?),
+            Value::Domain(d) => f(d)?,
+        })
     }
 }
 
@@ -808,7 +858,8 @@ pub fn serialize_nested_value<S, N, D: Domain>(v: &N, serializer: S) ->
 pub fn deserialize_nested_value<'de, D, N, Dom: Domain>(deserializer: D) ->
     Result<N, D::Error> where D: serde::Deserializer<'de>, N: NestedValue<Dom> + serde::Deserialize<'de>
 {
-    Ok(N::from_io_value(&super::magic::input_value(deserializer)?))
+    N::from_io_value(&super::magic::input_value(deserializer)?)
+        .map_err(|e| serde::de::Error::custom(format!("{:?}", e)))
 }
 
 //---------------------------------------------------------------------------
@@ -827,9 +878,9 @@ impl<N: NestedValue<D>, D: Domain> Annotations<N, D> {
     }
 
     pub fn maybe_slice(&self) -> Option<&[N]> {
-        match self.0 {
+        match &self.0 {
             None => None,
-            Some(ref b) => Some(&&b[..]),
+            Some(b) => Some(&b[..]),
         }
     }
 
@@ -847,16 +898,17 @@ impl<N: NestedValue<D>, D: Domain> Annotations<N, D> {
         self
     }
 
-    pub fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> Annotations<M, E>
+    pub fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> IOResult<Annotations<M, E>>
     where
-        F: Fn(&D) -> Value<M, E>
+        F: Fn(&D) -> IOResult<Value<M, E>>
     {
-        match self.0 {
+        Ok(match &self.0 {
             None =>
                 Annotations(None, PhantomData),
-            Some(ref b) =>
-                Annotations(Some(b.iter().map(|a| a.copy_via(f)).collect()), PhantomData),
-        }
+            Some(b) =>
+                Annotations(Some(b.iter().map(|a| a.copy_via(f)).collect::<Result<Vec<_>, _>>()?),
+                            PhantomData),
+        })
     }
 }
 
@@ -1005,8 +1057,6 @@ impl<'de, Dom: Domain> serde::Deserialize<'de> for RcValue<Dom> {
 
 //---------------------------------------------------------------------------
 
-use std::sync::Arc;
-
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub struct ArcValue<D: Domain>(Arc<AnnotatedValue<ArcValue<D>, D>>);
 
@@ -1060,8 +1110,8 @@ pub struct IOValue(Arc<AnnotatedValue<IOValue, IOValue>>);
 pub type UnwrappedIOValue = Value<IOValue, IOValue>;
 
 impl Domain for IOValue {
-    fn from_preserves(v: IOValue) -> Self {
-        v
+    fn from_preserves(v: IOValue) -> IOResult<Self> {
+        Ok(v)
     }
 
     fn as_preserves(&self) -> IOValue {
@@ -1106,8 +1156,8 @@ impl NestedValue<IOValue> for IOValue {
         self.clone()
     }
 
-    fn from_io_value(v: &IOValue) -> Self {
-        v.clone()
+    fn from_io_value(v: &IOValue) -> IOResult<Self> {
+        Ok(v.clone())
     }
 }
 
